@@ -35,6 +35,7 @@ terminating it.
 
 """
 
+import contextlib
 import os
 import socket
 import sys
@@ -360,6 +361,59 @@ class ComputeManager(manager.SchedulerDependentManager):
                            % locals())
                 raise exception.ImageTooLarge()
 
+        def _make_network_info():
+            if FLAGS.stub_network:
+                # TODO(tr3buchet) not really sure how this should be handled.
+                # virt requires network_info to be passed in but stub_network
+                # is enabled. Setting to [] for now will cause virt to skip
+                # all vif creation and network injection, maybe this is correct
+                network_info = []
+            else:
+                # NOTE(vish): This could be a cast because we don't do
+                # anything with the address currently, but I'm leaving it as a
+                # call to ensure that network setup completes.  We will
+                # eventually also need to save the address here.
+                network_info = self.network_api.allocate_for_instance(context,
+                                    instance, vpn=is_vpn,
+                                    requested_networks=requested_networks)
+                LOG.debug(_("instance network_info: |%s|"), network_info)
+            return network_info
+
+        def _make_block_device_info():
+            (swap, ephemerals,
+             block_device_mapping) = self._setup_block_device_mapping(
+                context, instance_id)
+            block_device_info = {
+                'root_device_name': instance['root_device_name'],
+                'swap': swap,
+                'ephemerals': ephemerals,
+                'block_device_mapping': block_device_mapping}
+            return block_device_info
+
+        def _deallocate_network():
+            if not FLAGS.stub_network:
+                LOG.debug(_("deallocating network for instance: %s"),
+                          instance['id'])
+                self.network_api.deallocate_for_instance(context,
+                                    instance)
+
+        def _cleanup():
+            with utils.save_and_reraise_exception():
+                self._instance_update(context,
+                                      instance_id,
+                                      vm_state=vm_states.ERROR)
+                if network_info is not None:
+                    _deallocate_network()
+
+        @contextlib.contextmanager
+        def _logging_error(instance_id, message):
+            try:
+                yield
+            except Exception as error:
+                with utils.save_and_reraise_exception():
+                    LOG.exception(_("Instance '%(instance_id)s' "
+                                   "failed %(message)s.") % locals())
+
         context = context.elevated()
         instance = self.db.instance_get(context, instance_id)
 
@@ -383,35 +437,16 @@ class ComputeManager(manager.SchedulerDependentManager):
 
         is_vpn = instance['image_ref'] == str(FLAGS.vpn_image_id)
         try:
-            # NOTE(vish): This could be a cast because we don't do anything
-            #             with the address currently, but I'm leaving it as
-            #             a call to ensure that network setup completes.  We
-            #             will eventually also need to save the address here.
-            if not FLAGS.stub_network:
-                network_info = self.network_api.allocate_for_instance(context,
-                                    instance, vpn=is_vpn,
-                                    requested_networks=requested_networks)
-                LOG.debug(_("instance network_info: |%s|"), network_info)
-            else:
-                # TODO(tr3buchet) not really sure how this should be handled.
-                # virt requires network_info to be passed in but stub_network
-                # is enabled. Setting to [] for now will cause virt to skip
-                # all vif creation and network injection, maybe this is correct
-                network_info = []
+            network_info = None
+            with _logging_error(instance_id, "network setup"):
+                network_info = _make_network_info()
 
             self._instance_update(context,
                                   instance_id,
                                   vm_state=vm_states.BUILDING,
                                   task_state=task_states.BLOCK_DEVICE_MAPPING)
-
-            (swap, ephemerals,
-             block_device_mapping) = self._setup_block_device_mapping(
-                context, instance_id)
-            block_device_info = {
-                'root_device_name': instance['root_device_name'],
-                'swap': swap,
-                'ephemerals': ephemerals,
-                'block_device_mapping': block_device_mapping}
+            with _logging_error(instance_id, "block device setup"):
+                block_device_info = _make_block_device_info()
 
             self._instance_update(context,
                                   instance_id,
@@ -419,23 +454,17 @@ class ComputeManager(manager.SchedulerDependentManager):
                                   task_state=task_states.SPAWNING)
 
             # TODO(vish) check to make sure the availability zone matches
-            try:
+            with _logging_error(instance_id, "failed to spawn"):
                 self.driver.spawn(context, instance,
                                   network_info, block_device_info)
-            except Exception as ex:  # pylint: disable=W0702
-                msg = _("Instance '%(instance_id)s' failed to spawn. Is "
-                        "virtualization enabled in the BIOS? Details: "
-                        "%(ex)s") % locals()
-                LOG.exception(msg)
-                return
 
             current_power_state = self._get_power_state(context, instance)
-            self._instance_update(context,
-                                  instance_id,
-                                  power_state=current_power_state,
-                                  vm_state=vm_states.ACTIVE,
-                                  task_state=None,
-                                  launched_at=utils.utcnow())
+            instance = self._instance_update(context,
+                                             instance_id,
+                                             power_state=current_power_state,
+                                             vm_state=vm_states.ACTIVE,
+                                             task_state=None,
+                                             launched_at=utils.utcnow())
 
             usage_info = utils.usage_from_instance(instance)
             notifier.notify('compute.%s' % self.host,
@@ -448,6 +477,8 @@ class ComputeManager(manager.SchedulerDependentManager):
             # deleted before it actually got created.  This should
             # be fixed once we have no-db-messaging
             pass
+        except Exception:
+            _cleanup()
 
     @exception.wrap_exception(notifier=notifier, publisher_id=publisher_id())
     def run_instance(self, context, instance_id, **kwargs):
@@ -652,8 +683,10 @@ class ComputeManager(manager.SchedulerDependentManager):
                        'instance: %(instance_id)s (state: %(state)s '
                        'expected: %(running)s)') % locals())
 
-        self.driver.snapshot(context, instance_ref, image_id)
-        self._instance_update(context, instance_id, task_state=None)
+        try:
+            self.driver.snapshot(context, instance_ref, image_id)
+        finally:
+            self._instance_update(context, instance_id, task_state=None)
 
         if image_type == 'snapshot' and rotation:
             raise exception.ImageRotationNotAllowed()
@@ -895,6 +928,7 @@ class ComputeManager(manager.SchedulerDependentManager):
         self._instance_update(context,
                               instance_ref["uuid"],
                               memory_mb=instance_type['memory_mb'],
+                              host=migration_ref['source_compute'],
                               vcpus=instance_type['vcpus'],
                               local_gb=instance_type['local_gb'],
                               instance_type_id=instance_type['id'])
@@ -939,10 +973,10 @@ class ComputeManager(manager.SchedulerDependentManager):
                 {'instance_uuid': instance_ref['uuid'],
                  'source_compute': instance_ref['host'],
                  'dest_compute': FLAGS.host,
-                 'dest_host':   self.driver.get_host_ip_addr(),
+                 'dest_host': self.driver.get_host_ip_addr(),
                  'old_instance_type_id': old_instance_type['id'],
                  'new_instance_type_id': instance_type_id,
-                 'status':      'pre-migrating'})
+                 'status': 'pre-migrating'})
 
         LOG.audit(_('instance %s: migrating'), instance_ref['uuid'],
                 context=context)
