@@ -57,8 +57,8 @@ def stubout_instance_snapshot(stubs):
     stubs.Set(vm_utils, 'wait_for_vhd_coalesce', fake_wait_for_vhd_coalesce)
 
 
-def stubout_session(stubs, cls):
-    """Stubs out two methods from XenAPISession"""
+def stubout_session(stubs, cls, product_version=None):
+    """Stubs out three methods from XenAPISession"""
     def fake_import(self):
         """Stubs out get_imported_xenapi of XenAPISession"""
         fake_module = 'nova.virt.xenapi.fake'
@@ -69,6 +69,10 @@ def stubout_session(stubs, cls):
                        lambda s, url: cls(url))
     stubs.Set(xenapi_conn.XenAPISession, 'get_imported_xenapi',
                        fake_import)
+    if product_version is None:
+        product_version = (5, 6, 2)
+    stubs.Set(xenapi_conn.XenAPISession, 'get_product_version',
+            lambda s: product_version)
 
 
 def stub_out_get_target(stubs):
@@ -282,9 +286,6 @@ class FakeSessionForMigrationTests(fake.SessionBase):
     def VDI_get_by_uuid(self, *args):
         return 'hurr'
 
-    def VDI_resize_online(self, *args):
-        pass
-
     def VM_start(self, _1, ref, _2, _3):
         vm = fake.get_record('VM', ref)
         if vm['power_state'] != 'Halted':
