@@ -13,6 +13,7 @@ function usage {
   echo "  -x, --stop               Stop running tests after the first error or failure."
   echo "  -f, --force              Force a clean re-build of the virtual environment. Useful when dependencies have been added."
   echo "  -p, --pep8               Just run pep8"
+  echo "  -P, --no-pep8            Don't run pep8"
   echo "  -c, --coverage           Generate coverage report"
   echo "  -h, --help               Print this usage message"
   echo "  --hide-elapsed           Don't print the elapsed time for each test along with slow test list"
@@ -32,13 +33,14 @@ function process_option {
     -n|--no-recreate-db) recreate_db=0;;
     -f|--force) force=1;;
     -p|--pep8) just_pep8=1;;
+    -P|--no-pep8) no_pep8=1;;
     -c|--coverage) coverage=1;;
     -*) noseopts="$noseopts $1";;
     *) noseargs="$noseargs $1"
   esac
 }
 
-venv=.nova-venv
+venv=.venv
 with_venv=tools/with_venv.sh
 always_venv=0
 never_venv=0
@@ -47,6 +49,7 @@ noseargs=
 noseopts=
 wrapper=""
 just_pep8=0
+no_pep8=0
 coverage=0
 recreate_db=1
 
@@ -86,8 +89,24 @@ function run_pep8 {
   srcfiles+=" `find tools/*`"
   srcfiles+=" nova setup.py plugins/xenserver/xenapi/etc/xapi.d/plugins/glance"
   # Just run PEP8 in current environment
+  #
+  # NOTE(sirp): W602 (deprecated 3-arg raise) is being ignored for the
+  # following reasons:
+  #
+  #  1. It's needed to preserve traceback information when re-raising
+  #     exceptions; this is needed b/c Eventlet will clear exceptions when
+  #     switching contexts.
+  #
+  #  2. There doesn't appear to be an alternative, "pep8-tool" compatible way of doing this
+  #     in Python 2 (in Python 3 `with_traceback` could be used).
+  #
+  #  3. Can find no corroborating evidence that this is deprecated in Python 2
+  #     other than what the PEP8 tool claims. It is deprecated in Python 3, so,
+  #     perhaps the mistake was thinking that the deprecation applied to Python 2
+  #     as well.
   ${wrapper} pep8 --repeat --show-pep8 --show-source \
-  --exclude=vcsversion.py ${srcfiles}
+    --ignore=E202,W602 \
+    --exclude=vcsversion.py ${srcfiles}
 }
 
 NOSETESTS="python run_tests.py $noseopts $noseargs"
@@ -139,7 +158,9 @@ run_tests
 # distinguish between options (noseopts), which begin with a '-', and
 # arguments (noseargs).
 if [ -z "$noseargs" ]; then
-  run_pep8
+  if [ $no_pep8 -eq 0 ]; then
+    run_pep8
+  fi
 fi
 
 if [ $coverage -eq 1 ]; then
